#include <p18cxxx.h>
#include <i2c.h>


/********************************************************************
*  C.Dupaty 06/2012 ralis  partir de i2Ceesr.c (MICROCHIP C18)
*     Function Name:    I2C echange                   
*     Return Value:     error condition status   
*     Parameters:       adresse du destinataire    
*						mode demande de donnes : (b7 de data =0)
*							donnee de configuration du destinataire (1 octet)
*							nombre d'octets  rcuprer du destinataire (max 255)
*							adresse du tableau ou seront ranges les donnes en provenance du destinataire
*						mode envoi de donnes : (b7 de data =1)
*							data	
*							nb, qui peut tre un parantre par exemple
*							adresse de la chaine de caractres
*   
*	 Prototype : 
*   unsigned char i2cEchange( unsigned char adresseI2c, unsigned char data, unsigned char nb, unsigned char *rdptr)
*
*     Description:      la fonction I2C_echange permet l'emission ou la reception de donnees sur bus I2C
*						le bit 7 du parametre data indique le sens de transfert des donnes
*						si b7 de data = 1 : envoie d'une chaine ASCII du maitre vers l'esclave
*							un pointeur contient l'adresse des octets  envoyer
*						si b7 de data =0 : demande au destinataire de lui envoyer n octets en rponse
*							les octets reus en rponse sont rangs dans le pointeur 
*
*				Exemple : i2cEchange(0x41,23,6,mon_tableau);  // c'est une demande de donnes (b7 de data =0)
*						envoie  l'adresse I2C 0x41 le paramre 23 et demande 6 octets en retour
*				Exemple : i2cEchange(0x41,0b10001010,6,mon_tableau);  // c'est un envoi de donnes (b7 de data =0), la valeur de nb (ici 6) n'est pas utilise
*						envoie  l'adresse I2C 0x41 la chaine ASCII mon_tableau
*
*  Remarque : le port I2C doit avoir t configur en mode maitre et SSPADD correctement configur.
* 				Exemple : 	OpenI2C(MASTER, SLEW_ON);// Initialize I2C module
*							SSPADD = 0x14;              //400kHz Baud clock(9) @32MHz
********************************************************************/
#if defined (I2C_V1) 

unsigned char i2cEchange( unsigned char adresseI2c, unsigned char data, unsigned char nb, unsigned char *rdptr)
{
  IdleI2C();                      // ensure module is idle
  StartI2C();                     // initiate START condition
  while ( SSPCON2bits.SEN );      // wait until start condition is over 
  if ( PIR2bits.BCLIF )           // test for bus collision
  {
    return ( -1 );                // return with Bus Collision error 
  }
  else
  {
    if ( WriteI2C( adresseI2c ) )    // envoie adresse destinataire (retourne 0 si pas d'erreur) 
    {
	  StopI2C(); 
      return ( -3 );              // set error for write collision
    }

    //IdleI2C();                    // ensure module is idle
    if ( !SSPCON2bits.ACKSTAT )   // test for ACK condition, if received
    {
      if ( WriteI2C( data ) )   // envoie data (retourne 0 si pas d'erreur) 
      {
	    StopI2C(); 
        return ( -3 );            // return with write collision error
      }
 
	if ( !SSPCON2bits.ACKSTAT )   // test for ACK condition, if received
    {
      if ( WriteI2C( nb ) )   // envoie nb (retourne 0 si pas d'erreur) 
      {
	    StopI2C(); 
        return ( -3 );            // return with write collision error
      }
 


      //IdleI2C();                  // ensure module is idle
      if ( !SSPCON2bits.ACKSTAT ) // test for ACK condition received
      {
///////////////////////////////////////////////////////////////////////////////////////////
			 if (!(data & 0b10000000))   // il s'agit d'une demande de donnees
			 {
			        RestartI2C();             // generate I2C bus restart condition
			        while ( SSPCON2bits.RSEN );  // wait until re-start condition is over 
			        if ( WriteI2C( adresseI2c +1 ) )// envoie adresse destinataire en lecture(retourne 0 si pas d'erreur) 
			        {
					  StopI2C(); 
			          return ( -3 );          // set error for write collision
			        }
			      
			        //IdleI2C();                // ensure module is idle
			        if ( !SSPCON2bits.ACKSTAT )// test for ACK condition received
			        {
			          if ( getsI2C( rdptr, nb ) )// lit nb octets et les range dans rdptr
			          {
					    return ( -1 );        // return with Bus Collision error
			          }
			
			          NotAckI2C();            // send not ACK condition
			          while ( SSPCON2bits.ACKEN );// wait until ACK sequence is over 
			          StopI2C();              // send STOP condition finale
			          while ( SSPCON2bits.PEN );// wait until stop condition is over 
			          if ( PIR2bits.BCLIF )   // test for bus collision
			          {
			            return ( -1 );        // return with Bus Collision error 
			          }
			        }
			        else
			        {
					  StopI2C();  
			          return ( -2 );          // return with Not Ack error
			        }
			 }
			 else   // il s'agit d'un envoi rdptr (ASCII)
			 {
		       	 if ( putsI2C ( rdptr ) )
		      	  {
					  StopI2C(); 
		     	     return( -4 );           // bus device responded possible error
		    	  }		
			//	  if ( !SSPCON2bits.ACKSTAT )   // test for ACK condition, if received
			//	    {
				      if ( WriteI2C( 0 ) )   // envoie 0 fin de message (retourne 0 si pas d'erreur) 
				      {
					    StopI2C(); 
				        return ( -3 );            // return with write collision error
				      }		  
				  while ( SSPCON2bits.PEN );      // wait until stop condition is over 
				  if ( PIR2bits.BCLIF )           // test for Bus collision
				  {
				    return ( -1 );                // return with Bus Collision error 
				  }
				  StopI2C(); 
				  return ( 0 );                   // return with no error 
			 }
/////////////////////////////////////////////////////////////////////////////////////////////
      }
      else
      {
	    StopI2C(); 
        return ( -2 );            // return with Not Ack error
      }
				} 
    }
    else
    {
	  StopI2C(); 
      return ( -2 );              // return with Not Ack error
    }
  }
  return ( 0 );                   // return with no error
}

#endif
